/* albin - Gestione completa alberi binari
 *
 * Copyright (C) 2003 Giuseppe Caulo <giusc@gbss.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glob.h"
#include "albin.h"
#include "randMT.h"

/* legge il prossimo carattere diverso da
   spazio nel file */
char readparenthesis(FILE *f)
{
  char c;

  c = getc(f);   /* legge il carattere */

  if (c == '\n')
    c = ' ';
  while (c != '(' && c != ')') {   /* se e' =' ' ne legge un altro */
    c = getc(f);
    if (c == '\n')
      c = ' ';
  }

  return c;
}  /* readparenthesis() */


DataTypePtr readsubtree(FILE *file_albero)
{
  char ch[3];
  char c;
  data i;
  DataTypePtr r;

  c = readparenthesis(file_albero);   /* legge la parentesi aperta */
  c = getc(file_albero);

  if (c == '\n')
    c = ' ';
  if (c == ')')
    return NULL;   /* se legge () allora l'albero e' vuoto */
  else {
    ch[0] = getc(file_albero);   /* altrimenti legge la radice */
    ch[1] = getc(file_albero);
    ch[2] = '\0';   /* altrimenti legge la radice */
    if (ch[0] == '\n') { i = ' '; }
    else if (ch[1] == '\n')  { i = ' '; }
    else { i = atoi(ch); }


    r = (DataTypePtr) malloc(sizeof(DataType));
    r->d = i;

    /* legge i sottoalberi */
    r->left = readsubtree(file_albero);
    r->right = readsubtree(file_albero);
    c = readparenthesis(file_albero);   /* legge la parentesi chiusa */

    return r;
  }
}  /* readsubtree() */


DataTypePtr loadtree(FILE *fd)
{
  int fsize;
  char *buf;
  char *pbuf;
  char cnum[10];
  int tmp, num;
  DataTypePtr nd = NULL;
  fseek(fd, 0L, SEEK_END);
  fsize = ftell(fd);
  fseek(fd, 0L, SEEK_SET);
  buf = malloc((fsize * sizeof(char)) + 1);
  fread(buf, 1, fsize, fd);
  buf[fsize] = '\0';
  /* now we can parse it */
  tmp = 0;
  for (pbuf = buf; *pbuf; pbuf++) {
	if (*pbuf >= '0' && *pbuf <= '9') {
	  if (tmp >= 10) { continue; }
	  cnum[tmp++] = *pbuf;
        } else if (tmp > 0) {
	  num = atoi(cnum);
	  tmp = 0;
	  if (!nd)
		create(&nd, num);
	  else
		insert(num, nd);

	}
  }
  return nd;
}


DataTypePtr readtree(char *nome_file)
{
  DataTypePtr result;
  FILE *file_albero;

  file_albero = fopen(nome_file, "r");
  if (!file_albero) {
    return NULL;
  }

/*  result = readsubtree(file_albero); */
  result = loadtree(file_albero);

  fclose(file_albero);

  return result;
}  /* readtree */



void create(node **root, data d) {
	node *n;
	if((n = malloc(sizeof(node))) == NULL) {
		fatal("Malloc Failed\n", 0);
	}
	n -> d = d;
	n -> parent = NULL;
	n -> left = NULL;
	n -> right = NULL;
	*root = n;
}

node *search(data d, node *r) {
	while(r != NULL && d != r -> d)
		if(d < r -> d)
			r = r -> left;
		else
			r = r -> right;
	return r;
}

void delete(node **root, node *r) {
	node *y, *x;
	if (!r) return;
	if(r->left==NULL || r->right==NULL)
		y = r;
	else y = successor(r);
	if(y -> left != NULL)
		x = y -> left;
	else x = y -> right;
	if(x != NULL)
		x -> parent = y -> parent;
	if(y -> parent == NULL)
		*root = x;
	else if(y == y -> parent -> left)
		y -> parent -> left = x;
	else y -> parent -> right = x;
	if(y != r)
		r -> d = y -> d;
	free(y);
} /* chiamata delete(&root, search(5, root)) */

void insert(data d, node *r) {
	node *n, *y;
	if(r == NULL) {
		create(&r, d);
		return;
	}
	if((n=malloc(sizeof(node)))==NULL) {
		fatal("Malloc Failed\n", 0);
	}
	n->d = d; n->left=n->right=NULL;
	y = NULL;
	while(r != NULL) {
		y = r;
		if(d <= r -> d)
			r = r -> left;
		else r = r -> right;
	}
	n -> parent = y;
	if(d <= y -> d)
		y -> left = n;
	else y -> right = n;
} /* chiamata: insert(4, root) */

node *maximum(node *r) {
	while(r -> right != NULL)
		r = r -> right;
	return r;
}

node *minimum(node *r) {
	while(r -> left != NULL)
		r = r -> left;
	return r;
}

node *predecessor(node *r) {
	node *y;
	if(r -> left != NULL)
		return maximum(r -> left);
	y = r -> parent;
	while(y!=NULL && r==y->left) {
		r = y;
		y = y -> parent;
	}
	return y;
}

node *successor(node *r) {
	node *y;
	if(r -> right != NULL)
		return minimum(r -> right);
	y = r -> parent;
	while(y!=NULL && r==y ->right) {
		r = y;
		y = y -> parent;
	}
	return y;
}
/* loop left , recurse right */
int count(DataTypePtr t)
{
   int n = 0;
   while (t != NULL)
   {
        n++;
        if (t->right != NULL)  n += count(t->right);
        t = t->left;
   }   return n;
}

DataTypePtr reverse( DataTypePtr t)
{
   DataTypePtr newNode = malloc( sizeof(node));

   newNode->left = NULL;
   newNode->right = NULL;
   newNode->d = t->d;

   if ( t->left)
      newNode->right = reverse( t->left);
   if ( t->right)
      newNode->left  = reverse( t->right);
   return newNode;
}

/* FUNZIONI DI STAMPA */
void print_tree(FILE *fd, DataTypePtr albero)
{
  if (albero == NULL) {
    fprintf(fd, "()");
    return;
  }
  fprintf(fd, "( %d ", albero->d);
  print_tree(fd, albero->left);
  print_tree(fd, albero->right);
  fprintf(fd, " )");
}  /* print_tree */

/* callback per scrittura in walk_* */
void do_print(DataTypePtr r)
{
	printf("%d ", r->d);
}

void do_free(DataTypePtr r)
{
/*	DataTypePtr *q;
	q = malloc(sizeof *q);
	*q = r;
	free(*q);
	*q = NULL;
	free(q);*/
	free(r);
	r = NULL;
}

#define WALK_DEBUG printf("Processing %p ( d = %d | right = %p | left = %p | parent = %p\n", \
			(void *)r, r->d, (void *)r->right, (void *)r->left, (void *)r->parent)

/* FUNZIONI ATTRAVERSAMENTO ARRAY */
/* inorder ricorsivo */
void walk_tree_ino(DataTypePtr r, void (*exec_func)(DataTypePtr))
{
	if (r != NULL)
	{
		walk_tree_ino(r->left, exec_func);
		exec_func(r);
		walk_tree_ino(r->right, exec_func);

	}
}
/* postorder ricorsivo */
void walk_tree_posto(DataTypePtr r, void (*exec_func)(DataTypePtr))
{
	if (r != NULL)
	{
		walk_tree_posto(r->left, exec_func);
		walk_tree_posto(r->right, exec_func);
		exec_func(r);
	}
}
/* preorder ricorsivo */
void walk_tree_preo(DataTypePtr r, void (*exec_func)(DataTypePtr))
{
	if (r != NULL)
	{
		exec_func(r);
		walk_tree_preo(r->left, exec_func);
		walk_tree_preo(r->right, exec_func);
	}
}

/* visita per livelli dell'albero (ver. iterativa) */
void walk_tree_iter(DataTypePtr albero, void (*exec_func)(DataTypePtr))
{
  stack coda;
  DataTypePtr a;

  reset(&coda);
  push(albero, &coda);

  while (! empty(&coda)) {
    a = pop(&coda);

    if (a != NULL) {
	  exec_func(albero);   /* analizza il nodo in cima alla coda */
	  if (a->left)
		  push(a->left, &coda);   /* mette in coda i sottoalberi */
	  if (a->right)
		  push(a->right, &coda);
    }
  }
}  /* print_tree_iter */

/* preorder iterativo */
void walk_tree_iter_preo(DataTypePtr albero, void (*exec_func)(DataTypePtr))
{
  stack coda;
  reset (&coda);
  while (True)
  {
    if (albero)
    {
      exec_func(albero);
      push(albero, &coda);
      albero = albero->left;
    }
    else
    {
      if (empty(&coda)) break;
      albero = pop(&coda);
      albero = albero->right;
    }
  }
}

/* inorder iterativo */
void walk_tree_iter_ino(DataTypePtr albero, void (*exec_func)(DataTypePtr))
{
  stack coda;

  reset(&coda);
  while (True) {
	  for (; albero; albero = albero->left)
	  {
		  push(albero, &coda);
	  }
	  if (!empty(&coda)) {
		  albero = pop(&coda);
		  exec_func(albero);
		  albero = albero->right;
	  } else { break; }
  }
}  /* print_tree_iter_ino */

/* postorder iterativo */
void walk_tree_iter_posto(DataTypePtr albero, void (*exec_func)(DataTypePtr))
{
  boolean comingUp = False;
  stack coda;

  reset(&coda);

  while (True)
  {
    if (!comingUp)
    {
      if (albero)
      {
        push(albero, &coda);
        albero = albero->left;
      }
      else
        comingUp = True;
    }
    else
    {
      DataTypePtr c = albero;

      if (empty(&coda)) break;
      albero = top(&coda);

      if (c == albero->left && albero->right)
      {
         albero = albero->right;
         comingUp = False;
      }
      else
      {
        exec_func(albero);
        pop(&coda);
      }
    }
  }
}

/* FUNZIONI DI STAMPA GRAFICA */

void oriz_indent(int n)
{
  int i;
  for (i = 0; i < n; i++) { printf(" "); }

}
void showtree(DataTypePtr t, int col)
{
   if (t != NULL)
   {
       oriz_indent(col); printf("%d\n", t->d); putchar('\n');
       showtree(t->right, col+1);
       showtree(t->left, col);
   }
}

void indent(int livello)
{
  int i;

  for (i = 1; i <= livello; i++)
    printf("  ");
}  /* Indenta() */


void printlevel(DataTypePtr albero, int livello)
{
  if (albero == NULL) {
    indent(livello);
    printf("()\n");
    return;
  }
  indent(livello);
  printf("( ");

  if (albero->left == NULL && albero->right == NULL) {
    printf("%d\n", albero->d);
    return;
  }

  printf("%d\n", albero->d);

  printlevel(albero->left, livello + 1);
  printlevel(albero->right, livello + 1);

  indent(livello);
  printf(")\n");
}  /* printlevel() */

void print_tree_indent(DataTypePtr albero)
{
  putchar('\n');
  printlevel(albero, 0);
}  /* print_tree_indent() */


int barre[100];     /* dice se ci vanno le barre verticali */

/* stampa le barre verticali prima del nodo */
void gindent( int livello )
{
	int i;
	for (i=0; i < livello; i++) {
		if( barre[i] )
			printf("|    ");
		else
			printf("     ");
	} /* gindent */
}


/* stampa il livello di un nodo, e se e' un
figlio destro o sinistro. quest'ultima informazione
e' necessaria per sapere se ci vanno le barre
verticali oppure no */

void printglevel( node *albero, int livello, int destro, int sinistro)
{
	if(albero!=NULL )
	{
		/* stampa del sottoalbero la destro barra ci va solo
		se il nodo corrente e' un figlio sinistro */
		if(albero->right!=NULL)
		{
			if( sinistro )
				barre[livello-1]=True;
			else
				barre[livello-1]=False;
			printglevel( albero->right , livello+1 , True , False );
		}

		/* stampa della radice */
		if(livello==0)
			printf("%d", albero->d);
		else
		{
			printf("   ");
			gindent(livello-1);
			if( destro )
				printf("+-");
			else
				printf("+-");
			printf("%d", albero->d);
		}

		if((albero->right != NULL) && (albero->left != NULL))
			printf("-+\n");
		else  if(albero->right != NULL)
			printf("-+\n");
		else  if(albero->left != NULL)
			printf("-+\n");
		else
			printf("\n");

		/* stampa del sottoalbero sinistro */
		if(( albero->left!=NULL ) )
		{
			if( destro )
				barre[livello-1]=True;
			else
				barre[livello-1]=False;
			printglevel( albero->left, livello+1 , False , True );
		}
	}
} /* printglevel() */

void print_tree_gindent( node *albero)
{
	int i;
	printf("\n");
	for (i=0; i< 100; i++)
		barre[i]=False;
	printglevel( albero, 0 , True , True );
} /* print_tree_gindent() */



/* IMPLEMENTAZIONE STACK */
void reset(stack *s) {
  s -> cnt = 0;
  s -> top = NULL;
}

void push(DataTypePtr d, stack *s) {
  item *p;
  if (!s) return;
  if((p = malloc(sizeof(item))) == NULL) {
    fatal("Malloc Failed\n", 0);
  }
  p -> d = d;
  p -> next = s -> top;
  s -> top = p;
  s -> cnt++;
}

DataTypePtr pop(stack *s) {
  DataTypePtr d;
  item *p;
  if (!s) return NULL;
  d = s -> top -> d;
  p = s -> top;
  s -> top = s -> top -> next;
  s -> cnt--;
  free(p);
  return d;
}

DataTypePtr top(stack *s) {
  if (!s) return NULL;
  return (s -> top -> d);
}

boolean empty(const stack *s) {
  return ((boolean) (s -> cnt == EMPTY));
}

boolean full(const stack *s) {
  return ((boolean) (s -> cnt == FULL));
}

