/* albin - Gestione completa alberi binari
 *
 * Copyright (C) 2003 Giuseppe Caulo <giusc@gbss.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ALBIN_H
#define _ALBIN_H

typedef struct node {
	data d;
	struct node *parent;
	struct node *left;
	struct node *right;
} node; /* nodo dell'albero */

typedef node DataType;
typedef node * DataTypePtr;

/* STACK */
#define EMPTY           0
#define FULL            10000

typedef struct item {
   DataTypePtr d;
   struct item *next;
} item; /* elemento dello stack */

typedef struct {
   int cnt;			/* contatore elementi*/
   item *top;           /* puntatore al top */
} stack;

/* PROTOTIPI STACK */
void reset(stack *);
void push(DataTypePtr, stack *);
DataTypePtr pop(stack *);
DataTypePtr top(stack *);
boolean empty(const stack *);
boolean full(const stack *);
/* PROTOTIPI ALBERO */
void create(node **, data);
node *search(data, node *);
node *minimum(node *);
node *maximum(node *);
node *successor(node *);
node *predecessor(node *);
void insert(data, node *);
void delete(node **, node *);
int count(DataTypePtr);
DataTypePtr reverse(DataTypePtr);
void do_print(DataTypePtr);
void do_free(DataTypePtr);
void walk_tree_ino(DataTypePtr, void (*exec_func)(DataTypePtr));
void walk_tree_preo(DataTypePtr, void (*exec_func)(DataTypePtr));
void walk_tree_posto(DataTypePtr, void (*exec_func)(DataTypePtr));
void walk_tree_iter(DataTypePtr, void (*exec_func)(DataTypePtr));
void walk_tree_iter_preo(DataTypePtr, void (*exec_func)(DataTypePtr));
void walk_tree_iter_ino(DataTypePtr, void (*exec_func)(DataTypePtr));
void walk_tree_iter_posto(DataTypePtr, void (*exec_func)(DataTypePtr));

DataTypePtr readtree(char *);
void print_tree(FILE *,DataTypePtr);
void showtree(DataTypePtr, int);
void print_tree_indent(DataTypePtr);
void print_tree_gindent(DataTypePtr);

#define WALK_TREE(what, mode, god, func) { \
		printf("%s_tree_%s()\n", #what, #mode); \
		walk_tree_##mode(god, func); \
		printf("\n"); \
	}
#define PRINT_TREE(mode, god) WALK_TREE(print, mode, god, do_print)
#define FREE_TREE(god) WALK_TREE(free, posto, god, do_free)


#endif /* _ALBIN_H */
