/* albin - Gestione completa alberi binari
 *
 * Copyright (C) 2003 Giuseppe Caulo <giusc@gbss.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glob.h"
#include "args.h"
#include "cmds.h"


static char **pargv;
static int pargc;

char *getword(char **rest)
{
  register char *o, *r;

  if (!rest)
    return *rest = "";
  o = *rest;
  while (*o && ((*o == ' ') || (*o == '\t')))
    o++;
  r = o;
  if (*o == '"') {
    r++;
    o = r;
    while (*o && ((*o != '"') && (*o != '\r') && (*o != '\n')) )
	    o++;
  }
  else if (*o == '\'') {
    r++;
    o = r;
    while (*o && ((*o != '\'') && (*o != '\r') && (*o != '\n')) )
	    o++;
  }
  else {
    while (*o && ((*o != ' ') && (*o != '\t') && (*o != '\r') && (*o != '\n')))
	    o++;
  }
  if (*o)
    *o++ = 0;
  *rest = o;
  return r;
}

void make_arg (char *line)
{
  char *q = NULL;
  pargc=0;
  do {
    q = getword(&line);
    if (!q || !(*q)) break;
    pargv[pargc++] = q;
    if (pargc == MAX_PARAMS)
	break;
  } while (q);
  pargv[pargc] = NULL;
  return;
}

int
argmatch (arg)
     char *arg;
{
  int i;
  size_t arglen;
  int matchind = -1;
  int ambiguous = 0;

  arglen = strlen (arg);

  for (i = 0; cmds[i].func != NULL; i++)
    {
      if (!strncmp (cmds[i].cmd, arg, arglen))
	{
	  if (strlen (cmds[i].cmd) == arglen)
	    /* Exact match found.  */
	    return i;
	  else if (matchind == -1)
	    /* First nonexact match found.  */
	    matchind = i;
	  else
	    /* Second nonexact match found.  */
	    ambiguous = 1;
	}
    }
  if (ambiguous)
    return -2;
  else
    return matchind;
}

void
invalid_arg (kind, value, problem)
     char *kind;
     char *value;
     int problem;
{
  if (problem == -1)
    fprintf (stderr, "invalid");
  else				/* Assume -2. */
    fprintf (stderr, "ambiguous");
  fprintf (stderr, " %s `%s'\n", kind, value);
}

void init_arg(void)
{
	pargv = (char **)malloc((MAX_PARAMS+1) * sizeof(char **));
	pargc = 0;
}

static void print_arg(void)
{
	int i;
	for (i = 0; i < MAX_PARAMS; i++)
	{
		printf("argv[%d] = %s\n", i, pargv[i]);
	}
}

int parse_command(char *line)
{
	int match;
	if (!(*line) || !line) {
		return 0;
	}
	make_arg(line);
	if (!pargv[0]) return 0;
	if ((match = argmatch(pargv[0])) < 0) {
		invalid_arg("command", pargv[0], match);
		return 0;
	} else {
		pargv[0] = cmds[match].cmd;
		cmds[match].func(pargc, pargv);
	}
	return 1;
}


