%{
#define YYSTYPE int
%}

%token NUMBER
%left '&' '|' '^'
%left '>' '<'
%left '+' '-' 
%left '*' '/' '%'
%left '~' UMINUS
%%
list:		/* null */
	|	list '\n'
	|	list expr 		{ output($2); }
	| 	list error '\n'		{ yyerrok; }
	;
expr:		NUMBER			{ $$ = $1; }
	|	expr '+' expr		{ $$ = $1 + $3; }
	|	expr '-' expr		{ $$ = $1 - $3; }
	|	expr '*' expr		{ $$ = $1 * $3; }
	|	expr '/' expr		{ if ($3 == 0)
						{ execerror("divide by zero error", ""); return 0; }
					  $$ = $1 / $3; }
	|	expr '%' expr		{ if ($3 == 0) 
						{ execerror("divide by zero error", ""); return 0; }
					  $$ = $1 % $3; }
	|	expr '&' expr		{ $$ = $1 & $3; }
	|	expr '|' expr		{ $$ = $1 | $3; }
	|	expr '^' expr		{ $$ = $1 ^ $3; }
	|	'(' expr ')'		{ $$ = $2; }
	| 	'-' expr %prec UMINUS	{ $$ = -$2; }
	|	'~' expr 		{ $$ = ~$2; }
	|	expr '>' expr		{ $$ = $1 >> $3; }
	|	expr '<' expr		{ $$ = $1 << $3; }
	;
%%
#include "glob.h"
#include "calc.h"

int yyparse(void);
void yyerror(char *);

static int warning(char s[], char t[]) {
	PRINT_ERR("err: %s\n", s);
	return 0;
}

static void output(int i) {
	PRINT("\t%d\n",i);
}

static void execerror(char s[], char t[]) {
	warning(s,t);
}

void yyerror(char s[]) {
	warning(s, "");
}


int vHexDigit[] = {
/*		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, A, B, C, D, E, F */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 0 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 1 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 2 */
/*		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', .... */
		0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, /* 3 */
/*		'A', 'B', 'C', 'D', 'E', 'F', .... */
		0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 4 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 5 */
/*		'a', 'b', 'c', 'd', 'e', 'f', .... */
		0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 6 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 7 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 8 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 9 */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* A */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* B */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* C */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* D */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* E */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* F */
};

int vHexToDec[] = {
/*		   0,   1, 2, 3, 4, 5, 6, 7, 8, 9, A, B, C, D, E, F */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 0 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 1 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 2 */
/*		 '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', .... */
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   0,   0,   0,   0,   0,   0, /* 3 */
/*		    , 'A', 'B', 'C', 'D', 'E', 'F', .... */
		   0,  10,  11,  12,  13,  14,  15,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 4 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 5 */
/*		    , 'a', 'b', 'c', 'd', 'e', 'f', .... */
		   0,  10,  11,  12,  13,  14,  15,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 6 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 7 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 8 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* 9 */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* A */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* B */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* C */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* D */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* E */
		   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, /* F */
};

unsigned int HexToInt(const char * ptr) { /* the string object have (const char *) cast operator... */
	unsigned int Dec = 0;

	while (vHexDigit[*ptr])
		Dec = 16 * Dec + vHexToDec[*ptr++];

	return Dec;
}
/*
int oatoi(const char *octal)
{
  register int i;

  if (!*octal)
    return -1;
  for (i = 0; ((*octal >= '0') && (*octal <= '7')); octal++)
    i = (i * 8) + (*octal - '0');
  if (*octal)
    return -1;
  return i;
}
*/
int oatoi(a)
char	*a;
{
	register int	r;
	register char	*p;
	register char	c;
	r = 0;
	p = a;
	while ((c = *p++) >= '0' && c <= '7')
		r = (r << 3) | (c &= 7);
	return (r);
}


char buf[CALC_BUFSIZE];
int bufp = 0;

int
getch (void)
{
  return buf[bufp++];
}

void ungetch(int c)
{
  --bufp;
}

int yylex() {
	int c;

	while ((c = getch()) == ' ' || c == '\t')
		;
	if (c == 0) return 0;
	if (c >= '0' && c <= '9') {
		yylval = c - '0';
		do
		{
		  c = getch();
		  if (c >= '0' && c <= '9') {
		    yylval = (yylval * 10) + (c - '0');
		  } else {
		    ungetch(c);
		    break;
		  }
		} while(1);
		return NUMBER;
	}
	return c;
}

static const char xerr[] = "cannot evaluate NULL! Can You?!\n";

void do_calc (int ac, char *av[]) {
	buf[0] = '\0';
	bufp = 0;
	if (!av[1]) {
	  PRINT_ERR("%s",xerr);
	  return;
	}
	strncpy(buf, av[1], sizeof(buf) -1);
	buf[strlen(buf)] = '\0';
	yyparse();
}

void do_hex(int ac, char *av[])
{
	int num;
	char *p;
	if (!av[1]) {
	  PRINT_ERR("%s",xerr);
	  return;
	}
	if (av[1][0] == 'x') {
		p = &av[1][1];
		num = (int)HexToInt(p);
	} else if (av[1][0] == '\\') {
		p = &av[1][1];
		num = oatoi(p);
	} else {
		num = atoi(av[1]);
	}
	PRINT("\t0x%x\n", num);
        return;
}

void do_oct(int ac, char *av[])
{
	int num;
	char *p;
	if (!av[1]) {
	  PRINT_ERR("%s",xerr);
	  return;
	}
	if (av[1][0] == 'x') {
		p = &av[1][1];
		num = (int)HexToInt(p);
	} else if (av[1][0] == '\\') {
		p = &av[1][1];
		num = oatoi(p);
	} else {
		num = atoi(av[1]);
	}
	PRINT("\t\\%.3o\n", num);
        return;
}

void do_dec(int ac, char *av[])
{
	int num;
	char *p;
	if (!av[1]) {
	  PRINT_ERR("%s",xerr);
	  return;
	}
	if (av[1][0] == 'x') {
		p = &av[1][1];
		num = (int)HexToInt(p);
	} else if (av[1][0] == '\\') {
		p = &av[1][1];
		num = oatoi(p);
	} else {
		num = atoi(av[1]);
	}
	PRINT("\t%d\n", num);
        return;
}

