/* albin - Gestione completa alberi binari
 *
 * Copyright (C) 2003 Giuseppe Caulo <giusc@gbss.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glob.h"
#include "cmds.h"
#include "lang.h"
#include "lang_cmds.h"
#include "randMT.h"
#include "albin.h"
#include "calc.h"

Cmd cmds[] = {
	{ "create", do_tree_create, HELP_CREATE },
	{ "load", do_tree_load, HELP_LOAD },
	{ "save", do_tree_save, HELP_SAVE },
	{ "print", do_tree_print, HELP_PRINT },
	{ "delete", do_tree_delete, HELP_DELETE },
	{ "clear", do_tree_clear, HELP_CLEAR },
	{ "calc", do_calc, HELP_CALC },
	{ "hex", do_hex, HELP_HEX },
	{ "oct", do_oct, HELP_OCT },
	{ "dec", do_dec, HELP_DEC },
	{ "cls", do_cls, HELP_CLS },
	{ "help", do_help, HELP_HELP },
	{ "version", do_version, HELP_VERSION },
	{ "copy", do_copy, HELP_COPY },
	{ "exit", do_quit, HELP_EXIT },
	{ "quit", do_quit, HELP_QUIT },
	{ NULL, NULL, -1 }
};


#define BADARGS(nl,nh,example) \
  if ((ac<(nl)) || (ac>(nh))) { \
    PRINT_ERR("Wrong args: should be \"%s\" %s\n",av[0], example); \
    return; \
  }


node *root = NULL; /* albero vuoto */

void do_tree_create(int ac, char *av[])
{
	int i = 0, j;
	BADARGS(2, 3, "[-r|-n <num>]");
#define HARD 10
#define SEX 99
	if (root != NULL) {
		PRINT("%s", GETMSG(ALREADY_EXISTS));
		return;
	}
	if (!strcmp(av[1], "-r")) {
	  i = (int)(randomMT() % 80);
	} else if (!strcmp(av[1], "-n")) {
	  if (!av[2]) {
	    return;
	  }
	  i = atoi(av[2]);
	}
	create(&root, GIVEME(HARD,SEX));
	for(j=0; j<i; j++)
	  insert(GIVEME(HARD,SEX), root);
	PRINT("%s", GETMSG(DONE));
}

void do_tree_load(int ac, char *av[])
{
	BADARGS(2,2,"<file>");
	if (root != NULL) {
		PRINT("%s", GETMSG(ALREADY_EXISTS));
		return;
	}
	if ((root = readtree(av[1])) == NULL) {
	  PRINT("%s", GETMSG(COULDNT_LOAD));
	  return;
	}
	PRINT("%s", GETMSG(DONE));
}

void do_tree_save(int ac, char *av[])
{
	FILE *fd;
	BADARGS(2,2,"<file>");
	if (root == NULL) {
	  PRINT("%s", GETMSG(TREE_NOT_EXISTS));
	  return;
	}
	if ((fd = fopen(av[1], "w+")) == NULL)
	{
	  PRINT("%s", GETMSG(FILE_WRITE_ERROR));
	  return;
	}
	print_tree(fd, root);
	fclose(fd);
	PRINT("%s", GETMSG(DONE));
}

void do_tree_clear(int ac, char *av[])
{
	if (root == NULL) {
	  PRINT("%s", GETMSG(TREE_NOT_EXISTS));
	  return;
	}
	PRINT("root at %p\n", (void *)root);
	FREE_TREE(root);
	root = NULL;
	PRINT("%s", GETMSG(DONE));
	return;
}

void do_tree_print(int ac, char *av[])
{
	DataTypePtr TheTree = NULL;
	int freeIT = 0;
	BADARGS(2, 3, "-<PRr|POr|PIr|PRi|POi|PIi|Gg|Gp|Pn> [-r]");
	if (root == NULL) {
		PRINT("%s", GETMSG(TREE_NOT_EXISTS));
		return;
	}
	if (av[2] && (!strcmp(av[2], "-r"))) {
		TheTree = reverse(root);
		freeIT = 1;
	} else {
		TheTree = root;
	}
	PRINT("Albero generato: nodi %d\n\n", count(root));
	SEP;
	if (!strcmp(av[1], "-PRr")) {
		PRINT_TREE(preo, TheTree);
	} else if (!strcmp(av[1], "-POr")) {
		PRINT_TREE(posto, TheTree);
	} else if (!strcmp(av[1], "-PIr")) {
		PRINT_TREE(ino, TheTree);
	}  else if (!strcmp(av[1], "-PRi")) {
		PRINT_TREE(iter_preo, TheTree);
	} else if (!strcmp(av[1], "-POi")) {
		PRINT_TREE(iter_posto, TheTree);
	} else if (!strcmp(av[1], "-PIi")) {
		PRINT_TREE(iter_ino, TheTree);
	} else if (!strcmp(av[1], "-Gg")) {
		print_tree_gindent(TheTree);
	} else if (!strcmp(av[1], "-Gp")) {
		print_tree_indent(TheTree);
	} else {
		print_tree(stdout, root);
		PRINT("\n");
	}
	if (freeIT) {
	  walk_tree_posto(TheTree, do_free);
	}
	SEP;
}

void do_tree_delete(int ac, char *av[])
{
	int u;
	node *n;
	BADARGS(2, 2, "<num>");
	if (root == NULL) {
		PRINT("%s", GETMSG(TREE_NOT_EXISTS));
		return;
	}
	u = atoi(av[1]);
	n = search(u, root);
	if (!n) {
		PRINT("%s", GETMSG(NODE_NOT_FOUND));
		return;
	}
	PRINT("Node found at %p\n", (void *)n);
	delete(&root, n);
	PRINT("%s", GETMSG(NODE_DELETED));
	return;
}

void do_cls(int ac, char *av[])
{
   printf("\033[0m");   //color 0
   printf("\033[2J");     //clear
   printf("\033[0;0H"); //gotoxy 0,0
}

void do_help(int ac, char *av[])
{
	int i=0;
	int found = 0;
	BADARGS(1,2, "[section]");
	PRINT("\n");
	if (!av[1]) {
	  PRINT("%s", GETMSG(MSG_COMMAND_LIST));
	  while(cmds[i].cmd != NULL)
	  {
	    printf("%.25s\n", cmds[i].cmd);
	    i++;
	  }
	} else {
	  while (cmds[i].cmd != NULL)
	  {
		  if (!strncmp(av[1],cmds[i].cmd, strlen(cmds[i].cmd))) {
			  printf("%s%.50s\n%s\n", GETMSG(MSG_COMMAND), cmds[i].cmd, GETCMDHELP(cmds[i].hm));
			  found++;
			  break;
		  }
		  i++;
	  }
	  if (!found)
	  {
		  PRINT("%s", GETMSG(MSG_SECTION_NOT_FOUND));
	  }
	}
	PRINT("\n");
}

void do_quit(int ac, char *av[])
{
        if (root != NULL) {
	  PRINT("%s", GETMSG(FREEING));
          walk_tree_posto(root, do_free);
          root = NULL;
          PRINT("%s", GETMSG(DONE));
        }
	PRINT("%s", GETMSG(GOODBYE));
	exit(0);
}

void do_version(int ac, char *av[])
{
	SEP;
	PRINT("Version Information: %s\n", VERSION);
	SEP;
}

void do_copy(int ac, char *av[])
{
	SEP;
	PRINT("%s\n", COPY);
	SEP;
}
